#include "config.h"

#include "date.h"
#include "hebrcal.h"

int Hebrew_Leap_Year(int year);
int Last_Month_Of_Hebrew_Year(int year);
int Last_Day_Of_Hebrew_Month(int month,
                              int year);
long Hebrew_Calendar_Elapsed_Days(int year);
int Days_In_Hebrew_Year(int year);
int Long_Heshvan(int year);
int Short_Kislev(int year);

int Hebrew_Leap_Year(int year)
{
  if ((((year*7)+1) % 19) < 7)
    return 1;
  else
    return 0;
}

int Last_Month_Of_Hebrew_Year(int year)
{
  if (Hebrew_Leap_Year(year))
    return 13;
  else
    return 12;
}

int Last_Day_Of_Hebrew_Month(int month, int year)
{
  if ((month == 2) ||
      (month == 4) ||
      (month == 6) ||
      (month == 10) ||
      (month == 13))
    return 29;
  if ((month == 12) && (!Hebrew_Leap_Year(year)))
    return 29;
  if ((month == 8) && (!Long_Heshvan(year)))
    return 29;
  if ((month == 9) && (Short_Kislev(year)))
    return 29;
  return 30;
}

long Hebrew_Calendar_Elapsed_Days(int year)
{
  long lValue;
  long lMonthsElapsed;
  long lPartsElapsed, lHoursElapsed;
  long lDay, lParts, lAlternativeDay;

  /* Months in complete cycles so far */
  lValue = 235 * ((long) (year-1) / 19);
  lMonthsElapsed = lValue;

  /* Regular months in this cycle */
  lValue = 12 * ((long) (year-1) % 19);
  lMonthsElapsed += lValue;

  /* Leap months this cycle */
  lValue = ((((long) (year-1) % 19) * 7) + 1) / 19;
  lMonthsElapsed += lValue;

  lPartsElapsed = (((lMonthsElapsed % 1080) * 793) + 204);
  lHoursElapsed = (5 +
                   (lMonthsElapsed * 12) +
                   ((lMonthsElapsed / 1080) * 793) +
                   (lPartsElapsed / 1080));

  /* Conjunction day */
  lDay = 1 + (29 * lMonthsElapsed) + (lHoursElapsed/24);

  /* Conjunction parts */
  lParts = ((lHoursElapsed % 24) * 1080) +
           (lPartsElapsed % 1080);

  /* If new moon is at or after midday, */
  if ((lParts >= 19440) ||

  /* ...or is on a Tuesday... */
      (((lDay % 7) == 2) &&
  /* at 9 hours, 204 parts or later */
       (lParts >= 9924)  &&
  /* of a common year */
       (!Hebrew_Leap_Year(year))) ||

  /* ...or is on a Monday at... */
      (((lDay % 7) == 1) &&
  /* 15 hours, 589 parts or later... */
       (lParts >= 16789) &&
  /* at the end of a leap year */
       (Hebrew_Leap_Year(year-1))))
  /* Then postpone Rosh HaShanah one day */
    lAlternativeDay = lDay+1;
  else
    lAlternativeDay = lDay;                                 

  /* If Rosh HaShanah would occur on Sunday, Wednesday, */
  /* or Friday */
  if (((lAlternativeDay % 7) == 0) ||
      ((lAlternativeDay % 7) == 3) ||
      ((lAlternativeDay % 7) == 5))
  /* Then postpone it one (more) day and return */
    lAlternativeDay++;

  return (lAlternativeDay);                
}

int Days_In_Hebrew_Year(int year)
{
  return ((long) Hebrew_Calendar_Elapsed_Days(year+1) -
          (long) Hebrew_Calendar_Elapsed_Days(year));
}

int Long_Heshvan(int year)
{
  if ((Days_In_Hebrew_Year(year) % 10) == 5)
    return 1;
  else
    return 0;
}

int Short_Kislev(int year)
{
  if ((Days_In_Hebrew_Year(year) % 10) == 3)
    return 1;
  else
    return 0;
}

long AbsoluteFromHebrew(calDate* date)
{
  long lValue, lReturnValue;
  int m;

  /* Days so far this month */
  lValue = date->day;
  lReturnValue = lValue;

  /* If before Tishri */
  if (date->month < 7)
  {
    /* Then add days in prior months this year before and */
    /* after Nisan. */
    for (m = 7; m <= Last_Month_Of_Hebrew_Year(date->year); m++)
    {
      lValue = Last_Day_Of_Hebrew_Month(m, date->year);
      lReturnValue += lValue;
    }
    for (m = 1; m < date->month; m++)
    {
      lValue = Last_Day_Of_Hebrew_Month(m, date->year);
      lReturnValue += lValue;
    }
  }
  else
  {
    for (m = 7; m < date->month; m++)
    {
      lValue = Last_Day_Of_Hebrew_Month(m, date->year);
      lReturnValue += lValue;
    }
  }

  /* Days in prior years */
  lValue = Hebrew_Calendar_Elapsed_Days(date->year);
  lReturnValue += lValue;

  /* Days elapsed before absolute date 1 */
  lValue = 1373429;
  lReturnValue -= lValue;

  return (lReturnValue);
}

void HebrewFromAbsolute(long lDate, calDate* date)
{
  int approx, y, m, year, month, day;
  long temp;
  calDate mydate;
  int uStart;

  /* Approximation */
  approx = (lDate+1373429) / 366;

  /* Search forward from the approximation */
  y = approx;
  for (;;)
  {
    mydate.day   = 1;
    mydate.month = 7;
    mydate.year  = y+1;
    temp = AbsoluteFromHebrew(&mydate);
    if (lDate < temp) break;
    y++;
  }
  year = y;

  /* Starting month for search for month */
  mydate.day   = 1;
  mydate.month = 1;
  mydate.year  = year;
  temp = AbsoluteFromHebrew(&mydate);
  if (lDate < temp)
    uStart = 7;
  else
    uStart = 1;

  /* Search forward from either Tishri or Nisan */
  m = uStart;
  for (;;)
  {
    mydate.day   = Last_Day_Of_Hebrew_Month(m, year);
    mydate.month = m;
    mydate.year  = year;
    temp = AbsoluteFromHebrew(&mydate);
    if (lDate <= temp)
      break;
    m++;
  }
  month = m;

  /* Calculate the day by subtraction */
  mydate.day   = 1;
  mydate.month = month;
  mydate.year  = year;
  temp = AbsoluteFromHebrew(&mydate);
  day = lDate-temp+1;

  date->day   = day;
  date->month = month;
  date->year  = year;
}

